/******************************************************************************
 *                                                                            *
 *   (C) 1993,94 by Radicon Ltd.                                              *
 *   Define variable                                                          *
 *   11-18-92 12:55pm                                                         *
 *                                                                            *
 ******************************************************************************/

#include "dos.h"
#include "windows.h"
#include "dfkisl.h"
#include "prkmpt1.h"

#define HD

#ifdef HD
  extern LPSTR pTime;
#else
  extern unsigned char bufmsg[];  //in kisl1.c deklariert
#endif

/******************************************************************************
 *          --- int tr_message ---                                            *
 *  Nachricht (Funktionscode und Daten) zum Controller uebertragen            *
 *                                                                            *
 *  Parameter:  int fun           - Funktionscode (Nachricht)                 *
 *              int rd            - Portnummer des Datenregisters             *
 *              int rs            - Portnummer des Steuerregisters            *
 *              unsigned char *p  - Zeiger auf Datenpuffer                    *
 *              int l             - Anzahl der zu uebertragenden Bytes        *
 *                                                                            *
 *  Rueckgabewert:  0   - OK                                                  *
 *                 -1   - FEHLER                                              *
 *                 -2   - Controller kann die angeforderte Funktion zu diesem *
 *                        Zeitpunkt nicht ausfuehren                          *
 *                        oder unbekannter Funktionscode                      * 
 *                 -3   - Checksummenfehler                                   *
 ******************************************************************************/

#ifdef DEBUG
  int retspot_tr;
#endif

int tr_message(int fun, int rd, int rs, unsigned char *p, int l)
{
  register int i,j,k;
  int w;
  //unsigned char *p = bufmsg;  //???

//Datenregister einlesen...zu welchem Zweck ???
//eingelesener Wert wird nicht weiter verwendet...
#ifndef __WIN32__
  _asm 
  {
    mov dx,rd;
    in  al,dx;
  }
#endif
//inp(rd);  //zum Assemblercode aequivalente Windowsfunktion

  //Funktionscode an Controller uebermitteln 
  i = out_byte(rd,rs,fun);

  //Fehler bei Uebertragung => Funktion mit FEHLER beenden
  if (i == -1)
  {
#ifdef DEBUG
      retspot_tr = 1;
#endif

    return -1;
  }

  //Antwort des Controllers einlesen
  i = in_byte(rd,rs,&w);

  //Fehler bei Uebertragung => Funktion mit FEHLER beenden
  if (i == -1)
  {
#ifdef DEBUG
      retspot_tr = 2;
#endif

    return -1;
  }

  //Controllerantwort != ACK => Fehler spezifizieren...
  if (w != ACK) 
  { //???
    if (fun == CFF && w == PROGR)
    {
#ifdef DEBUG
        retspot_tr = 0;
#endif

      return 0;
    }

    //Controllerantwort = NAK => Funktion mit Fehlercode -2 beenden
    if (w == NAK)
    {
#ifdef DEBUG
        retspot_tr = 3;
#endif

      return -2;
    }

    //unspezifische Controllerantwort => Funktion mit FEHLER beenden
#ifdef DEBUG
      retspot_tr = 4;
#endif

    return -1;
  }

  //Cheksumme mit 0 initialisieren
  k = 0;

  //Datenpuffer byteweise zum Controller uebertragen...
  for (i = 0; i < l; i++)
  { //Byte aus dem Datenpuffer zum Controller uebertragen
    j = out_byte(rd,rs,*p);

    //Fehler bei Uebertragung => Funktion mit FEHLER beenden
    if (j == -1)
    {
#ifdef DEBUG
        retspot_tr = 5;
#endif

      return -1;
    }

    //gesendete Bytes aufsummieren (fuer Checksumme)
    k += *p;
    
    //Wenn ein zu uebertragendes Byte den gleichen Wert wie der Steuercode DLE
    //besitzt, so mu dieses Byte zweimal hintereinander uebertragen werden,
    //um die Steuercodebedeutung aufzuheben
    if (*p == 0x10) //*p == DLE
    { //zweites senden des Bytes
      j = out_byte(rd,rs,*p);

      //Fehler bei Uebertragung => Funktion mit FEHLER beenden
      if (j == -1)
      {
#ifdef DEBUG
          retspot_tr = 6;
#endif

        return -1;
      }

      //gesendete Bytes aufsummieren (fuer Checksumme) 
      k += *p;
    }

    //Pufferzeiger inkrementieren
    p++;
  }

  //Nachricht DLE zum Controller uebertragen => Daten vollstaendig uebertragen 
  j = out_byte(rd,rs,DLE);

  //Fehler bei Uebertrageung => Funktion mit FEHLER beenden 
  if (j == -1)
  {
#ifdef DEBUG
      retspot_tr = 7;
#endif

    return -1;
  }

  //gesendet Bytes aufsummieren (fuer Checksumme)
  k += DLE;

  //Nachricht EOT zum Controller uebertragen
  //=> Nachrichtenuebertragung abschliessen
  j = out_byte(rd,rs,EOT);

  //Fehler bei Uebertragung => Funktion mit FEHLER beenden
  if (j == -1)
  {
#ifdef DEBUG
      retspot_tr = 8;
#endif

    return -1;
  }

  k += EOT; //gesendete Bytes aufsummieren (fuer Checksumme)
  k = ~k;   //bitweise Negation (1er-Komplement)
  k++;      //=> 2er-Komplement

  //ermittelte Checksumme zum Controller uebertragen
  j = out_byte(rd,rs,k);

  //Fehler bei Uebertragung => Funktion mit FEHLER beenden
  if (j == -1)
  {
#ifdef DEBUG
      retspot_tr = 9;
#endif

    return -1;
  }

  //Antwort des Controllers einlesen
  i = in_byte(rd,rs,&w);

  //Fehler bei Uebertragung => Funktion mit FEHLER beenden
  if (i == -1)
  {
#ifdef DEBUG
      retspot_tr = 10;
#endif

    return -1;
  }

  //Controllerantwort = NAK (Checksummenfehler)
  //=> Funktion mit CHECKSUMMENFEHLER beenden
  if (w == NAK)
  {
#ifdef DEBUG
      retspot_tr = 11;
#endif

    return -3;
  }

#ifdef DEBUG
    retspot_tr = 0;
#endif

  return 0; //Funktion mit OK beenden
}


/******************************************************************************
 *          --- int rc_message ---                                            *
 *  Nachricht vom Controller empfangen                                        *
 *  (Funktionscode dem Controller uebermitteln und Ergebnisdaten empfangen)   *
 *                                                                            *
 *  Parameter:  int fun           - Funktionscode (Nachricht)                 *
 *              int rd            - Portnummer des Datenregisters             *
 *              int rs            - Portnummer des Statusregisters            *
 *              unsigned char *p  - Zeiger auf Datenpuffer                    *
 *              int ml            - max. Laenge der zu empfangenden Daten     *
 *              int *l            - Zeiger auf Anzahl der empfangenen Daten   *
 *                                                                            *
 *  Bemerkung: Der Wert von *l beim Funktionsaufruf ist unerheblich, da *l    *
 *             im Verlauf der Funktion mit der tatsaechlichen Anzahl der      *
 *             empfangenen Byte ueberschrieben wird.                          *
 *                                                                            *
 *  Rueckgabewert:  0   - OK                                                  *
 *                 -1   - FEHLER                                              *
 *                 -2   - Controller kann die angeforderte Funktion zu diesem *
 *                        Zeitpunkt nicht ausfuehren                          *
 *                        oder unbekannter Funktionscode                      *
 *                 -3   - Checksummenfehler                                   *
 *                 -4   - Daten stehen nicht bereit                           *
 ******************************************************************************/

#ifdef DEBUG
  int retspot_rc;
#endif

int rc_message(int fun, int rd, int rs, unsigned char *p,int ml, int *l)
{
  register int i,j;
  int w1,w2;

//Datenregister einlesen...zu welchem Zweck ???
//eingelesener Wert wird nicht weiter verwendet...
#ifndef __WIN32__
  _asm 
  {
    mov dx,rd;
    in al,dx;
  }
#endif
//inp(rd);  //zum Assemblercode quivalente Windowsfunktion

  //Funktionscode zum Controller uebertragen
  i = out_byte(rd,rs,fun);

  //Fehler bei Uebertragung => Funktion mit FEHLER beenden
  if (i == -1)
  {
#ifdef DEBUG
      retspot_rc = 1;
#endif

    return i; //(i=-1)
  }

  //Antwort des Controllers empfangen
  i = in_byte(rd,rs,&w1);

  //Fehler bei Uebertragung => Funktion mit FEHLER beenden
  if (i == -1)
  {
#ifdef DEBUG
      retspot_rc = 2;
#endif

    return i;
  }

  //Controllerantwort != ACK => Fehler spezifizieren...
  if (w1 != ACK) 
  { //Controllerantwort = CAN (Kommunikation abbrechen) 
    //=> Funktion mit Fehlercode -4 beenden
    if (w1 == CAN)
    {
#ifdef DEBUG
        retspot_rc = 3;
#endif

      return -4;
    }

    //Controllerantwort = NAK (unbekannter Funktionscode)
    //=> Funktion mit Fehlercode -2 beenden
    if (w1 == NAK)
    {
#ifdef DEBUG
        retspot_rc = 4;
#endif

      return -2;
    }

    //unspezifische Controllerantwort => Funktion mit FEHLER beenden
#ifdef DEBUG
      retspot_rc = 5;
#endif

    return -1;
  }

  j = 0;  //Checksumme mit 0 initialisieren
  *l = 0; //Anzahl der empfangenen Daten mit 0 initialisieren

  //Daten byteweise vom Controller empfangen bis die max. Anzahl erreicht ist
  //oder die Datenuebertragung durch DLE vom Controller beendet wird
  while ((*l) < ml) 
  { //Byte vom Controller empfangen 
    i = in_byte(rd,rs,&w1);

    //Fehler bei Uebertragung => Funktion mit FEHLER beenden
    if (i == -1)
    {
#ifdef DEBUG
          retspot_rc = 6;
#endif

        return -1;
    }

    //empfangene Bytes aufsummieren (fuer Checksumme) 
    j += w1;

    //empfangenes Byte = DLE => testen ob Steuercode(DLE) oder Datenwert(10h)
    if (w1 == DLE) 
     {  //Folgebyte vom Controller empfangen
        i = in_byte(rd,rs,&w2);

        //Fehler bei Uebertragung => Funktion mit FEHLER beenden 
        if (i == -1)
        {
#ifdef DEBUG
            retspot_rc = 7;
#endif

          return -1;
        }

        //empfangene Bytes aufsummieren (fuer Checksumme)
        j += w2;

        //empfangenes Byte = EOT (Ende der Uebertragung) => vorheriges Byte
        //war Steuercode DLE => Datenempfang vorzeitig abbrechen
        if (w2 == EOT)
          break;

        //empfangenes Byte != DLE => vorheriges Byte war weder Steuercode(DLE)
        //noch Datenwert(10h) => Funktion mit FEHLER beenden
        if (w2 != DLE)
        {
#ifdef DEBUG
            retspot_rc = 8;
#endif

          return -1;
        }
    }

    //empfangenes Byte in den Datenpuffer uebernehmen 
    *p = w1;
    //Pufferzeiger inkrementieren
    p++;
    //Anzahl der empfangenen Daten inkrementieren
    (*l)++;
  }

  //letztes Byte vom Controller empfangen (zur Ueberpruefung der Checksumme)
  i = in_byte(rd,rs,&w1);

  //Fehler bei Uebertragung => Funktion mit FEHLER beenden
  if (i == -1)
  {
#ifdef DEBUG
      retspot_rc = 9; 
#endif

    return -1;
  }

  //Checksumme pruefen
  j = (j + w1) & 0xff;

  //Checksumme != 0 => NAK (wegen Checksummenfehler) an den Controller senden
  //und Funktion mit CHECKSUMMENFEHLER beenden
  if (j != 0) 
  { //Nachricht NAK (Checksummenfehler) an den Controller uebermitteln
    i = out_byte(rd,rs,NAK);

    //Fehler bei Uebertragung => Funktion mit FEHLER beenden
    if (i == -1)
    {
#ifdef DEBUG
        retspot_rc = 10; 
#endif

      return -1;
    }

#ifdef DEBUG
      retspot_rc = 11;
#endif

    //Funktion mit Fehlercode -3 beenden
    return -3;
  }

  //Nachricht ACK (Checksumme OK) an Controller uebermitteln
  i = out_byte(rd,rs,ACK);

  //Fehler bei Ubertragung => Funktion mit FEHLER beenden
  if (i == -1)
  {
#ifdef DEBUG
      retspot_rc = 12; 
#endif

    return -1;
  }

#ifdef DEBUG
    retspot_rc = 0;
#endif

  return 0; //Funktion mit OK beenden
}



#ifdef DEBUG
  int retspot_out;
#endif

/******************************************************************************
 *           --- int out_byte ---                                             *
 *  Datenbyte zum Controller uebertragen                                      *
 *                                                                            *
 *  Parameter:  int rd  - Portnummer des Datenregisters                       *
 *              int rs  - Portnummer des Steuerregisters                      *
 *              int d   - Datenpuffer (1 Integer)                             *
 *                                                                            *
 *  Rueckgabewert:    0 - OK                                                  *
 *                   -1 - FEHLER                                              *
 ******************************************************************************/
int out_byte(int rd, int rs, int d)
{
  register int i,j,k;

  //(1) aktuelle Systemzeit ermitteln
  i = j = crnt_time();

  //Warten auf 'Bereit-Zustand' des Controllers innerhalb einer bestimmten Zeit
  //(TIMEOUT-Schleife)
  while ((j-i) < TMOUT)
  { //Status des Controllers ermitteln
    k = status_wr(rs);
 
    //Controller bereit => TIMEOUT-Schleife abbrechen
    if (k == 0) 
      break;

    //(2) aktuelle Systemzeit ermitteln
    j = crnt_time();

    //Fehler: zuletzt gemessene Systemzeit(2) liegt vor der ersten gemessenen
    //Systemzeit(1) => Anpassen der ersten Systemzeit(1)
    //=> bisher verstrichene Zeit wird zurueckgesetzt => TIMEOUT beginnt erneut
    if (j < i) 
      i=j;
  }

  //Controller nicht bereit => Funktion mit FEHLER beenden
  if (k != 0)
  {
#ifdef DEBUG
      retspot_out = 1;
#endif

    return -1;
  }

  //Byte zum Controller uebertragen (Datenregister schreiben)
#ifndef __WIN32__
    _asm 
    {
      mov dx,rd;  //Portadresse des Datenregisters in dx schreiben
      mov ax,d;   //zu uebertragendes Byte in ax schreiben
      out dx,al;  //Byte in das Datenregister des Controllers schreiben
    }
#endif
//outp(rd,d); //zum Assemblercode aequivalente Windowsfunktion

#ifdef DEBUG
    retspot_out = 0;
#endif

  return 0; //Funktion mit OK beenden
}


#ifdef DEBUG
  int retspot_in;
#endif

/******************************************************************************
 *            --- int in_byte ---                                             *
 *  Datenbyte vom Controller empfangen                                        *
 *                                                                            *
 *  Parameter:  int rd  - Portnummer des Datenregisters                       *
 *              int rs  - Portnummer des Steuerregisters                      *
 *              int d   - Zeiger auf Datenpuffer (1 Integer)                  *
 *                                                                            *
 *  Rueckgabewert:   0   - OK                                                 *
 *                  -1   - FEHLER                                             *
 ******************************************************************************/
int in_byte(int rd, int rs, int *d)
{
  register int i,j,k;

  //(1) aktuelle Systemzeit ermitteln
  i = j = crnt_time();

  //Warten auf 'Bereit-Zustand' des Controllers innerhalb einer bestimmten Zeit
  //(TIMEOUT-Schleife)
  while (j-i < TMOUT) 
  { //Controllerstatus ermitteln
    k = status_rd(rs);

    //Daten stehen bereit => TIMEOUT-Schleife abbrechen
    if (k != 0) 
      break;

    //(2) aktuelle Systemzeit ermitteln
    j = crnt_time();

    //Fehler: zuletzt gemessene Systemzeit(2) liegt vor der ersten gemessenen
    //Systemzeit(1) => Anpassen der ersten Systemzeit(1)
    //=> bisher verstrichene Zeit wird zurueckgesetzt => TIMEOUT beginnt erneut
    if (j < i) 
      i = j;
  }

  //Daten stehen nicht bereit => Funktion mit FEHLER beenden
  if (k == 0)
  {
#ifdef DEBUG
      retspot_in = 1;
#endif

    return -1;
  }

  //Byte vom Controller empfangen (Datenregister lesen)
#ifndef __WIN32__
    _asm 
    {
      sub ax,ax   //Register ax loeschen
      mov dx,rd   //Portadresse des Datenregisters in dx schreiben
      in  al,dx   //Inhalt des Datenregisters nach al einlesen
      mov i,ax    //Inhalt des Datenregisters in der Variablen i speichern
    }
#endif
  //ausgelesenes Byte in den Puffer schreiben
  *d = i;
//*d=inp(rd); //zum Assemblercode aequivalente Windowsfunktion

#ifdef DEBUG
    retspot_in = 0;
#endif

  return 0; //Funktion mit OK beenden 
}


/******************************************************************************
 *           --- int status_rd ---                                            *
 *  Testen ob Controller zur Datenuebergabe (senden) bereit                   *
 *                                                                            *
 *  Parameter:  int rs  - Portnummer des Steuerregisters                      *
 *                                                                            *
 *  Rueckgabewert:  0   - Controller nicht breit (Datenregister leer)         *
 *                  !0  - Controller bereit (Datenregister nicht leer)        *
 ******************************************************************************/
int status_rd(int rs)
{ //Steuerregister lesen und auswerten...
#ifndef __WIN32__
    _asm
    {
      mov dx,rs       //Portadresse des Steuerregisters in dx schreiben
      in  al,dx       //Inhalt des Steuerregisters einlesen
      and ax,MSKREAD  //eingelesenen Status auswerten
    }
#endif

  return _AX; //Wo ist das Symbol _AX deklariert ???
              //Vermutung: BORLAND C spezifisch (CPU-Register AX)
}

/******************************************************************************
 *           --- int status_wr ---                                            *
 *  Testen ob Controller zur Datenuebernahme (empfangen) bereit               *
 *                                                                            *
 *  Parameter:  int rs  - Portnummer des Steuerregisters                      *
 *                                                                            *
 *  Rueckgabewert: 0   - Controller bereit (Datenregister leer)               *
 *                 !0  - Controller nicht bereit (Datenregister nicht leer)   *
 ******************************************************************************/
int status_wr(int rs)
{ //Steuerregister lesen und Status auswerten...
#ifndef __WIN32__
    _asm
    {
      mov dx,rs       //Portadresse des Steuerregisters in dx schreiben
      in  al,dx       //Inhalt des Steuerregisters einlesen
      and ax,MSKWRITE //eingelesenen Status auswerten
    }
#endif

  return _AX; //Wo ist das Symbol _AX deklariert ???
              //Vermutung: BORLAND C spezifisch (CPU-Register AX)
}


/******************************************************************************
 *            --- int crnt_time ---                                           *
 *  Zeit seit dem Start des Systems in Millisekunden ermitteln                *
 *                                                                            *
 *  Implementation abhaengig vom Symbol HD                                    *
 *                                                                            *
 *  HD definiert:       Rueckgabewert:  int - ???                             *
 *                                                                            *
 *  HD nicht definiert: Rueckgabewert:  dword - Systemzeit in Millisekunden   *
 ******************************************************************************/
#ifdef HD
  int crnt_time()
  { //Was steht in *pTime ???
    return *pTime;
  }
#else
  DWORD crnt_time()
  {//Windowssystemfunktion: Systemzeit in Millisekunden
    return timeGetTime()
  }
#endif
// __LastLine__