#include    "rc_def.h"
#include    "comhead.h"

#include    "m_devcom.h"
#include    "m_layer.h"


#include    "m_xscan.h"
#include    "c_layer.h"

extern TMain    Main;
extern LPDList  lpDList;
extern LPDataBase lpDBase;


//*****************************************************************************
//**********         Klasse TAreaScanCCDParameters ****************************
//*****************************************************************************

TAreaScanCCDParameters::TAreaScanCCDParameters()
{
  char Ident[] = "AreaScanCCD";
  char buf[ MaxString ];
  double dDistance;
  SetDevice( dlGetIdByName( CCDDevice ) );
  fMaxTime = 6.0;
  fDelay = 0.0;
  eSaveFormat = StandardFile;
  bSaveOnReady = FALSE;
  nOmega = mlGetIdByName( Omega );
  //! Omegamotor und zugehoerige Werte setzen
  mlSetAxis( nOmega );
  dOmegaWidth = mGetValue( Width );
  if ( !mGetDistance( dDistance ) )
  {
         dOmegaMin = 40.0;
         dOmegaMax = 46.0;
  }
  else
  {
         dOmegaMin = dDistance;
         dOmegaMax = dDistance + 4.0;
  }

  //! einlesen der Daten aus dem [AreaScanCCD]-Abschnitt des Ini-Files
  GetPrivateProfileString( Ident, "ExposureTime", "1.0", buf, MaxString,
                GetCFile() );
  if ( CreateDefaults() )
         WritePrivateProfileString( Ident, "ExposureTime", buf, GetCFile() );
  fMaxTime = atof( buf );

  GetPrivateProfileString( Ident, "Delay", "0.0", buf, MaxString,
                GetCFile() );
  if ( CreateDefaults() )
         WritePrivateProfileString( Ident, "Delay", buf, GetCFile() );
  fDelay = atof( buf );

  GetPrivateProfileString( Ident, "Mode", "0", buf, MaxString,
                GetCFile() );
  if ( CreateDefaults() )
         WritePrivateProfileString( Ident, "Mode", buf, GetCFile() );
  iHardwareMode = atoi( buf );

};

//! setzt den Detektor, dessen id-nummer dem Param. id entspricht
//! dabei wird bCCDDevice gesetzt, wenn der Detektor ein CCD ist
void TAreaScanCCDParameters::SetDevice( int id )
{
  Device = lpDList->DP( id );
  bCCDDevice = ( Device->GetId() == dlGetIdByName( CCDDevice ) );
};


//*****************************************************************************
//**********         Klasse TAreaScanCCD **************************************
//*****************************************************************************

TAreaScanCCD::TAreaScanCCD( void )
{
  /* TPLOTTEST START
  bMeasurementActive = FALSE;
  // TPLOTTEST END
    */

  // Fuer TPlotData
  LowerBound = 0.1;
  UpperBound = 10000.0;
  bXY_Scaling        = TRUE;

  TMDIWindow::RegisterWnd();
  bMeasurementActive = FALSE;
  //! erstellen der menueeintraege des Areascanfensters
  TheMenu = CreatePopupMenu();
  AppendMenu( TheMenu, MF_STRING, cm_SetupAreaScanCCD, "Setup &Measurement..." );
  AppendMenu( TheMenu, MF_STRING, cm_ShowCurveParam, "Visualising &Options..." );
  AppendMenu( TheMenu, MF_STRING, cm_InitializeAreaScanCCD, "Start &AreaScanCCD" );

  lpDBase->New();
  nActiveScan = 0;
  MainCurve = lpDBase->GetCurve( nActiveScan );
  };


TAreaScanCCD::~TAreaScanCCD()
{
  char        buf[ MaxString ];
  RECT        rc, rcm;

  //! Hauptfensterverweis auf areascanfenster loeschen + datenbasis loeschen
  Main.hAreaScanCCDWindow = NULL;
  //! loeschen des Fenstermenue + senden von Detektorsignalen an das
  //! AreaScanfenster beenden
  DestroyMenu( TheMenu );
  //! AreaScanfenster-eigenschaften sowie Datenpfad,Messzeit und SaveOnReady in
  //! Ini-File ablegen
  GetWindowRect( GetFrameHandle(), &rcm );
  GetWindowRect( hWndChild, &rc );
  sprintf( buf, "%d", rc.left - rcm.left - 4 );
  WritePrivateProfileString( "AreaScanCCD", "xo", buf, GetCFile() );
  sprintf( buf, "%d", rc.top - rcm.top - 42 );
  WritePrivateProfileString( "AreaScanCCD", "yo", buf, GetCFile() );
  sprintf( buf, "%d", rc.right - rc.left );
  WritePrivateProfileString( "AreaScanCCD", "dx", buf, GetCFile() );
  sprintf( buf, "%d", rc.bottom - rc.top );
  WritePrivateProfileString( "AreaScanCCD", "dy", buf, GetCFile() );
  WritePrivateProfileString( "AreaScanCCD", "DataPath", szDataPath, GetCFile() );
  sprintf( buf, "%.2f", fMaxTime );
  WritePrivateProfileString( "AreaScanCCD", "ExposureTime", buf, GetCFile() );
  sprintf( buf, "%d", bSaveOnReady );
  WritePrivateProfileString( "AreaScanCCD", "SaveOnReady", buf, GetCFile() );
}


//! loescht aktuelle datenbasis + baut AreaScanfenster neu auf
//! Rueckkehrcode immer True
BOOL TAreaScanCCD::New( void )
{
  lpDBase->New();
  nActiveScan = 0;
  MainCurve = lpDBase->GetCurve( nActiveScan );
  return TPlotData::New();
};


//! Darstellen des AreaScanCCD-Fenster
void TAreaScanCCD::Create( HWND hwnd )
{
  RECT        rc;

  hWndChild = hwnd;
  Main.hAreaScanCCDWindow = hwnd;
  Main.AreaScanCCDWindow = this;
  Delay( 2000 );
  //! einlesen der Fenstereigenschaften, des Datenpfades aus dem
  //! [AreaScanCCD]-Abschnitt des Ini-Files
  rc.left = GetPrivateProfileInt( "AreaScanCCD", "xo", 500, GetCFile() );
  rc.top = GetPrivateProfileInt( "AreaScanCCD", "yo", 400, GetCFile() );
  rc.right = rc.left + GetPrivateProfileInt( "AreaScanCCD", "dx", 170, GetCFile() );
  rc.bottom = rc.top + GetPrivateProfileInt( "AreaScanCCD", "dy", 50, GetCFile() );
  GetPrivateProfileString( "AreaScanCCD", "DataPath", "c:\\", szDataPath,
                MaxString, GetCFile() );
  bSaveOnReady = GetPrivateProfileInt( "AreaScanCCD", "SaveOnReady", 0, GetCFile() );

  if ( szDataPath[ strlen( szDataPath ) - 1 ] != '\\' )
         strcat( szDataPath, "\\" );
  //! setzen der Filter und der Extension
  strcpy( szFilter, "CCD File (*.ccd)|*.ccd||" );
  strcpy( szExtension, "ccd" );
  //! Titelzeile des Fensters bestimmen + aktualisieren des Hauptmenues +
  //! darstellen des Scanfensters

  SetTitle();
  UpdateMenu();
  SetWindowPos( hWndChild, NULL, rc.left, rc.top, rc.right - rc.left,
                                         rc.bottom - rc.top, SWP_NOZORDER );

  }

BOOL TAreaScanCCD::SetTitle()
{
  char aTitle[ MaxString ];

strcpy( aTitle, "AreaScanCCD:" );

  //! haengt "<untitled.ccd>" bei neuer Datei , sonst FileName an
  //! und schreibt ihn ins Fenster
  if ( bIsNewFile )
         strcat( aTitle, "<untitled.ccd>" );
  else
         strcat( aTitle, FileName );
  SetWindowText( hWndChild, aTitle );
  return TRUE;
};

//***************** Mausbedienung *********************************************

//! aufruf wenn rechte Maustaste gedrueckt
//! Fenstermenue an aktueller Mausposition ausgeben
//! eingabeparam. aktuelle Mauspos.low(8) x-pos
//!                                high(8) y-pos.
void TAreaScanCCD::rButtonDown( LONG lParam )
{
  POINT        pp;
  pp.x = LOWORD( lParam );
  pp.y = HIWORD( lParam );
  ClientToScreen( hWndChild, &pp );
  TrackPopupMenu( TheMenu, 0, pp.x, pp.y, 0, hWndChild, NULL );
};

//! behandelt die Kommandos 'Setup Measurement...' und 'Inquire Data'
//! des Kindfenster-menue ; aufruf aus m_main.cpp  DoCommandsChild()
void TAreaScanCCD::InitializeDlg( WPARAM task, LPARAM )
{
  TModalDlg * dlg;

  switch ( task )
  {
					 //! aufruf der Dialogbox 'Einstellungen AreaScanCCD'
			case cm_SetupAreaScanCCD:
					 dlg = ( TSetupAreaScanCCD* ) new TSetupAreaScanCCD( this );
					 //! zeichnet DialogBox (in dlg_tpl.cpp)
					 dlg->ExecuteDialog( hWndChild );
					 delete dlg;
					 break;
  }
};

//! wird aufgerufen aus m_main.cpp bei cm_InitializeScan
//! startet den Scanvorgang
void TAreaScanCCD::InitializeTask( WPARAM, LPARAM )
{
  char buf[ 101 ];
  char fmt[ MaxString ];
#ifdef GermanVersion
  char        HeadLine[] = "Start AreaScan";
#else
  char        HeadLine[] = "Starting AreaScan";
#endif
  char msg[ 101 ];
  sprintf(msg,"Delay: %f \t Hardwaremode: %d",fDelay,iHardwareMode);
  MessageBox( hWndChild, msg, HeadLine, MBSTOP );
  return ;
}

//*****************************************************************************
//******* Dialog zum Setzen der AreaScanCCD-Parameter ****************************
//*****************************************************************************

//! Konstruktor fuer Dialogeinstellungen AreaScan
//! dabei wird zunaechst der Konstruktor fuer Dialogboxen mit dem String, der
//! die Ressource spezifiziert, aufgerufen
#ifdef GermanVersion
TSetupAreaScanCCD::TSetupAreaScanCCD( TAreaScanCCD* scan ) :
         TModalDlg( "AreaScanCCDParameters" )
#else
TSetupAreaScanCCD::TSetupAreaScanCCD( TAreaScanCCD* scan ) :
         TModalDlg( "AreaScanCCDParametersEng" )
#endif
{
  //! einige Parameter vom AreaScan-fenster uebernehmen
  Scan        = scan;
  Device = Scan->Device;
};

#pragma argsused
//! setzt Defaultwerte der Dialogbox bei Aufruf der Box
//! Rueckkehrcode immer true
BOOL TSetupAreaScanCCD::Dlg_OnInit( HWND hwnd, HWND hwndCtl, LPARAM lParam )
{
  int i = 0;
  char buf[ MaxString ];

  //! Werte fuer Checkboxen 'Save on Ready' setzen
  CheckDlgButton( hwnd, id_SaveOnReady, Scan->bSaveOnReady );
  //CheckDlgButton( hwnd, id_SaveOnReady, 1 );

  //! diverse Werte in Dialogbox setzen
  sprintf( buf, "%.1f", Scan->fMaxTime );
  SetDlgItemText( GetHandle(), id_MaxTime, buf );

  SetDlgItemText( GetHandle(), id_Unit, mGetAxisUnit() );
  sprintf( buf, mGetDF(), Scan->dOmegaMax );
  SetDlgItemText( GetHandle(), id_AngleMax, buf );
  sprintf( buf, mGetDF(), Scan->dOmegaMin );
  SetDlgItemText( GetHandle(), id_AngleMin, buf );
  CheckDlgButton( GetHandle(), id_Standard,
                Scan->eSaveFormat == StandardFile );
  CheckDlgButton( GetHandle(), id_DIB_Format, Scan->eSaveFormat == Bitmap );
  //! aufnehmen aller verfuegbaren Detektoren in Combobox und vorauswahl des
  //! aktuell eingestellten Det.
  hDeviceList = GetDlgItem( GetHandle(), id_ChooseDevice );
  i = 0;
  while ( i <= lpDList->LastId() )
         ComboBox_AddString( hDeviceList, lpDList->DP( i++ ) ->Characteristic );
  ComboBox_SelectString( hDeviceList, 0, lpDList->DP( 0 )->Characteristic );

  //! bewirkt aufruf von Dlg_OnCommand mit den Kommandos cm_ParamSet und
  //! cm_SetupParameters
  FORWARD_WM_COMMAND( hwnd, cm_ParamSet, hwndCtl, 0, SendMessage );
  FORWARD_WM_COMMAND( hwnd, cm_SetupParameters, hwndCtl, 0, SendMessage );
  return TRUE;
};


//! behandelt Kommandos in der Dialogbox
void TSetupAreaScanCCD::Dlg_OnCommand( HWND hwnd, int id, HWND hwndCtl,
         UINT codeNotify )
{
  char buf[ MaxString ];

  //! abhaengig von Dialogelement bzw. Kommando
  switch ( id )
  {
                //! initialisierung
         case cm_ParamSet:
                //! diverse Dialogelemente setzen (Fehler)
                SetDlgItemText( hwnd, id_Catalog, Scan->szDataPath );
                //! Focus auf OK-Button der Box
                SetFocus( GetDlgItem( hwnd, IDOK ) );
                break;

                //! 'Update Parameters'-Button betaetigt
         case cm_SetupParameters:
                FORWARD_WM_COMMAND( hwnd, cm_InquireRelevantData, hwndCtl, 0,
                         SendMessage );
                FORWARD_WM_COMMAND( hwnd, cm_ParamSet, hwndCtl, 0, SendMessage );
                break;

                //! uebernehmen von einigen Dialogeintraegen ohne pruefung (fehler)
                //! 'Data Catalog','Save each Scan',Omegaschrittweite, Speicherformat
         case cm_InquireRelevantData:


                // bei 'Cancel' bleibt womglich inkorrekter Pfad gespeichert !!!


                GetDlgItemText( GetHandle(), id_Catalog, ( LPSTR ) Scan->szDataPath,
                         MaxString );
                // Bestimmen der Schrittweite und Schrittrelation
                GetDlgItemText( GetHandle(), id_AngleWidth, buf, MaxString );
                Scan->dOmegaWidth = atof( buf );

                Scan->eSaveFormat = StandardFile;
                if ( IsDlgButtonChecked( GetHandle(), id_DIB_Format ) )
                  Scan->eSaveFormat = Bitmap;
                if ( IsDlgButtonChecked( GetHandle(), id_Standard ) )
                  Scan->eSaveFormat = StandardFile;
                break;

                //! anclicken der folgenden Dialogelemente bewirkt uebernahme der akt.
                //! Einstellungen ohne pruefung
         case id_DIB_Format:
  case id_FixThetaAxis: case id_Standard:
                FORWARD_WM_COMMAND( GetHandle(), cm_SetupParameters, 0, 0, PostMessage );
                break;

                //! hier werden IDOK und IDCANCEL ... behandelt
         default:
                TModalDlg::Dlg_OnCommand( hwnd, id, hwndCtl, codeNotify );
  }
};

//! realisiert test der Dialogboxinhalte auf ihre konsistenz vor verlassen der
//! Dialogbox (durch OK-Button)
//! Rueckkehrcode false wenn die Dialogbox nicht geschlossen werden kann (werte
//! inkonsistent) sonst true
BOOL TSetupAreaScanCCD::CanClose( void )
{
  char buf[ MaxString ], dir[ MAXDIR ], drv[ MAXDRIVE ], name[ MAXFILE ], ext[ MAXEXT ];
  OFSTRUCT of;
  int hFile;
  float valueF;
  BOOL bFailure = FALSE;

  //! uebernehmen von einigen werten ohne pruefung
  FORWARD_WM_COMMAND( GetHandle(), cm_SetupParameters, 0, 0, SendMessage );
  //! uebernahme der CheckButtons 'Save on Ready'
  Scan->bSaveOnReady = IsDlgButtonChecked( GetHandle(), id_SaveOnReady );

  //! uebernahme der Messzeit mit Pruefung, ob der Wert ungleich 0 (Fehler)
  //! wenn nicht, wird Box wird nicht verlassen
  GetDlgItemText( GetHandle(), id_MaxTime, buf, MaxString );
  valueF = atof( buf );
  if ( 0.0 != valueF )
  {
         Scan->fMaxTime = valueF;
  }
  else
         bFailure = TRUE;

  //! uebernahme des Datenpfades, wenn er sich beschreiben laesst
  //! ansonsten auf "C:\\" setzen + Box wird nicht verlassen
  GetDlgItemText( GetHandle(), id_Catalog, ( LPSTR ) Scan->szDataPath,
                MaxString );
  // Testen, ob sich der Pfad auch beschreiben laesst
  if ( Scan->szDataPath[ strlen( Scan->szDataPath ) - 1 ] != '\\' )
         strcat( Scan->szDataPath, "\\" );
  fnsplit( Scan->szDataPath, drv, dir, name, ext );
  fnmerge( Scan->szDataPath, drv, dir, "check", "tmp" );
  hFile = OpenFile( Scan->szDataPath, &of, OF_CREATE );
  if ( -1 != hFile )
  {
         _lclose( hFile );
         remove( Scan->szDataPath );
         fnmerge( Scan->szDataPath, drv, dir, "", "" );
  }
  else
  {
         bFailure = TRUE;
         strcpy( Scan->szDataPath, "c:\\" );
  }
  //! bei aufgetretenen Fehlern cm_ParamSet senden und Dialogbox nicht
  //! schliessen
  if ( bFailure )
  {
         PostMessage( GetHandle(), WM_COMMAND, ( WPARAM ) cm_ParamSet, 0l );
         return FALSE;
  }
  return TRUE;
};


