#ifndef   __M_CURVE_H
#define   __M_CURVE_H
// (C) Copyright 1993,95 by Heiko Damerow MPG AG "Roentgenbeugung"

//! von diffr.gruppe kommentiert

#if defined (Build_Library)
#define _CURVECLASS _export
#elif defined(Use_Library)
#define _CURVECLASS _import
#else
#define _CURVECLASS
#endif


//! gibt an nach welchem Kriterium die Kurve sortiert ist
typedef enum
{
  X_Sorted = 1, Y_Sorted, Z_Sorted, Incoming
} TOrder;

//! Fehlertype bei DatenBasis
typedef enum
{
  //! max. Anzahl speicherbarer Kurven ist erreicht
  MaxScanExceeded = 1,
  //! kein ausreich. Speicherplatz verfuegbar
  NoMemory
} TCFault;


//! Struktur eines Elements der Kurve
struct CPoint
{
  float Pt[ 3 ];
  BOOL Valid;
};

typedef CPoint * LPCPoint;

//! Klasse zum Verwalten einer Folge von Messwerten und zugehoeriger Parametern
class __rtti _CURVECLASS TCurve
{
  public:

         //! Konstuktor Parameter gibt Anzahl der Messpunkte der Kurve an die
         //! 'vorlaeufig' geplant sind (kann spaeter noch geaendert werden)
         TCurve( int );
         ~TCurve( void );

         //! haengt einen Punkt(tripel) an Kurve an
         //! Rueckkehrcode false wenn fehler, wenn erfolgreich true
         BOOL PAdd( float &x, float &y, float &z );

         //! liest ein Wertetripel der Kurve aus
         //! Rueckkehrcode false wenn nicht moeglich sonst true
         BOOL PGet( float &x, float &y, float &z );

         //! wie PAdd (um schneller hintereinander Punkte anzuhaengen)
         //! Rueckkehrcode false bei Fehler, sonst true
         BOOL FastPAdd( float &x, float &y, float &z );

         //! wie PGet (fuer schnellen Zugriff)
         //! Rueckkehrcode false bei Fehler(Kurve hat keine Punkte,
         //!                     Speicherfehler,...)
         //! sonst true
         BOOL FastPGet( float &x, float &y, float &z );

         //! 'oeffnet' Kurve, macht Zugriffe auf Punkte erst moeglich
         //! Rueckkehrcode false bei Fehler sonst true
         BOOL FastOpen( void );

         //! 'schliesst' Kurve, kein Zugriff auf die Punkte moeglich
         //! Rueckkehrcode immer true
         BOOL FastClose( void );

         //! addiert val zum Y-wert des akt. Punktes der Kurve
         //! Rueckkehrcode false bei Fehler sonst true
         //! (wird nicht benutzt)
         BOOL ValueAdd( float &val );

         //! geht in der Kurve einen Punkt zurueck
         void BackStep( void );

         //! setzt Kurve auf Initialwerte zurueck
         //! Rueckkehrcode immer true
         BOOL New( void );

         //! markiert alle Punkte der Kurve als ungueltig, deren Y-wert<=(Level*100)
         //! Prozent von maximaler intensitaet der Kurve sind
         //! Rueckkehrcode false bei fehler sonst true
         //! (nur von GetPeakProperties benutzt)
         BOOL DeleteUnderGround( float Level );

         //! markiert alle Punkte der Kurve als ungueltig, laesst nur lokal.
         //! Max. uebrig (scheinbar nicht fertig impl.)
         //! Rueckkehrcode false bei Fehler sonst true
         //! (wird nicht benutzt)
         BOOL DeleteFlanks();

         //! legt alle Punkte(x,y,z) der Kurve in Datei mit angegeb. Namen
         //! (Parameter) ab
         //! Rueckkehrcode false bei fehler sonst true
         //! (wird nicht benutzt)
         BOOL Save( LPCSTR );

         //! speichert Kurve in ang. Datei(1.Parameter), 2. Param. gibt Art der
         //! Speicherung an(nur eine Option impl., diese speichert y-werte),
         //! 3. Param. nicht benutzt
         //! Rueckkehrcode false bei Fehler sonst true
         BOOL Save( LPCSTR, int, int );

         //! (nicht impl.)
         void SetLastPoint( void );

         //! setzt achse, nach welcher die Kurve geordnet ist
         //! (x,y,z oder messreihenf.)
         void SetOrder( TOrder order )
         { eOrder = order; };

         //! setzt internen zeiger auf den (pos). Wert der Kurve falls moeglich
         //! bei unzulaess. pos auf 1. bzw. letzten Wert
         //! Rueckkehrcode false wenn Kurve keine Punkte hat sonst true
         BOOL SetPP( int pos );

         //! liefert Id (Nr.) der Kurve als Rueckkehrcode
         //! (wird nicht benutzt)
         int GetId( void )
         { return Id; };

         //! liefert akt. Pos. des internen Zeiger der Kurve als Rueckkehrcode
         //! (wird nicht benutzt)
         int GetPP( void )
         { return ActualIdx; };

         //! liefert Anzahl der akt. im Speicher befindl. Kurven als Rueckkehrcode
         //! (wird nicht benutzt)
         int GetLastId( void )
         { return LastId; };

         //! liefert die Anzahl der Punkte der Kurve als Rueckkehrcode
         int GetPNumber( void )
         { return MaxPointIdx + 1; };

         //! berechnet eine durch den letzen Param. bestimmte Fkt
         //! (max. Intensitaet/Schwerpunkt)
         //! 1.Param(ausg.) x enthaelt nach aufruf je nach funktion berechneten Wert
         //! 2.Param(ausg.) y enthaelt nach aufruf je nach funktion berechneten Wert
         //! 3.Param(ausg.) z enthaelt nach aufruf je nach funktion berechneten Wert
         //! 4.Param(eing.) func ermoeglicht verschieden funktionen aufzurufen
         //!                (nur 2 implementiert)
         //!                  func=2  dann x,y,z tripel ermitteln, fuer das y (die
         //!                          Intensitaet) maximal ist
         //!                  func<>2 dann x,y,z Schwerpunkt der Kurve ermitteln
         //!                          (wird scheinbar nicht benutzt)
         //! Rueckkehrcode false wenn fehler sonst true
         BOOL GetGravityCenter( float&, float&, float&, int );

         //! liefert den in der Kurve vorhandenen dem Param 'search' naehsten X-wert
         //! (den indexwert) als Rueckkehrcode zurueck und den zugehoerigen Y-Wert
         //! sowie den abstand zwischen X-wert und search
         //! 1.param (eingabe) search in Kurve soll nach einem X-punkt gesucht
         //!                   werden, der zu search den geringsten abstand besitzt
         //! 2.Param (ausgabe) delta enthaelt nach aufruf den Abstand von search
         //!                   zum naehsten X-wert
         //! 3.Param (ausgabe) value enthaelt nach aufruf den zum naehsten X-wert
         //!                   gehoerigen Y-wert(Intensitaet)
         //! Rueckkehrcode ist der Index fuer den gilt X[Index] ist naehster X-wert
         //! zu search, wenn search im inneren des x-bereichs der Kurve ist sonst
         //! (wenn ausserhalb dieses bereiches oder bei fehler) den Wert 0
         //! voraussetzung ist, dass die X-werte der kurve von klein nach gross
         //! sortiert sind
         int GetValueByValue( float, float&, float& );

         //! nicht von diffr.gruppe auskommentiert
         //! GetReflection wird nirgends benutzt
         //! BOOL       GetReflection( int order, LPCPoint );

         //! liefert die Breite des 1.Bereiches, in dem alle y-werte des Tripels
         //! die Haelfte des Intensitaetbereichs(Y(max)-Y(min)) uebersteigen, sowie
         //! aus diesem Bereich x-wert,y-wert des Tripels fuer das Y-wert
         //! (Intensitaet) maximal ist
         //! 1. bereich bedeutet der am weitesten links liegende Bereich der obige
         //!     Eigenschaften erfuellt
         //! 1.Param(ausgabe) hw enthaelt nach aufruf breite des Bereichs
         //! 2.Param(ausgabe) pos enthaelt nach aufruf X-wert an der max. y-wert
         //!                      (intensitaet) in diesem Bereich
         //! 3.Param(ausgabe) intens enthaelt nach aufruf max.y-wert(intensitaet)
         //!                         in diesem Bereich
         //! Rueckkehrcode false wenn fehler sonst true
         //! anmerkung: dieser Bereich darf nicht bis ans ende der Kurve reichen
         //! d.h. kurve darf nicht monoton wachsend sein (sonst return false)
         //! (wird nicht benutzt)
         BOOL GetPeakProperties( float&, float&, float& );

         //! zuweisungsoperator fuer kurven, kopiert alle x,y,z,v werte sowie die
         //! max. und min. fuer x,y,z
         TCurve& operator=( const TCurve& );

         //! enthalten die min und max werte  der 3 achsen [0=x,1=y,2=z]
         //! bei kurve ohne punkte max[0..2]=-(min[0..2]=1000000.0)
         float Min[ 3 ], Max[ 3 ];

  private:

         //! handle auf speicherobjekt (x-werte)
         HGLOBAL hMemX;

         //! handle auf speicherobjekt (y-werte)
         HGLOBAL hMemY;

         //! handle auf speicherobjekt (z-werte)
         HGLOBAL hMemZ;

         //! handle auf speicherobjekt (valid)
         HGLOBAL hMemV;

         //! ordnung in der kurve vorliegt X_sorted/Y_sorted/Z_sorted oder Incoming
         TOrder eOrder;

         //! zeigt an ob kurve gerade im speicher fix. und somit zugriffe moeglich
         //! sind
         BOOL bStreamOpen;
         int Id;             // bezeichnet Kurve
         BOOL bEnd;           // Ende-Erkennung
         static int LastId;         // Anzahl der genutzten Kurven im Programm
         int MaxPointIdx;    // indiziert den letzten Eintrag
         int MaxMemIdx;      // indiziert den maximal moeglichen Eintrag
         int ActualIdx;      // indiziert den aktuellen Eintrag
         float* lpPX;           // Zeiger auf Datenfeld von X
         float* lpPY;           // Zeiger auf Datenfeld von Y
         float* lpPZ;           // Zeiger auf Datenfeld von Z
         BOOL* lpPV;           // Zeiger auf Datenfeld von Validate
         float* lpFastPX;       // Zeiger auf Datenfeld von X
         float* lpFastPY;       // Zeiger auf Datenfeld von Y
         float* lpFastPZ;       // Zeiger auf Datenfeld von Z

         //! zeigt an ob xyz-tripel gleichen Indezes gueltig ist
         BOOL* lpFastPV;       // Zeiger auf Datenfeld von Validate

}
;

typedef TCurve FAR * LPCurve;

//! klasse zum verwalten von mehreren Kurven (areascan)
class __rtti _CURVECLASS TDataBase
{
  public:
         TDataBase( void );
         ~TDataBase( void );

         //! setzt internen Kurvenindex auf 0
         void Reset( void )
         { nActualIdx = 0; };

         //! bestimmt Anzahl der Kurven
         //! Rueckkehrcode = Anzahl der Kurven
         int GetCNumber( void )
         { return nMaxCurveIdx + 1; };

         //! liefert die Kurve als Rueckkehrcode deren nummer gleich dem
         //! uebergebenen Param. ist
         LPCurve GetCurve( int );

         //!? liefert als Rueckkehrcode die Scanreportcurve
         LPCurve GetScanReport( void )
         { return ScanReport; };

         //! liefert als Rueckkehrcode akt. Kurvenindex
         int GetCIdx( void )
         { return nActualIdx; };

         //! setzt Kurvenindex auf angegebenen wert(Parameter)
         void SetCIdx( int );

         //! fuegt eine Kurve der angegebenen Groesse zur Datenbasis hinzu
         //! Rueckkehrcode false wenn max. Kurvenanzahl erreicht sonst true
         BOOL AddCurve( int size );

         //! loescht alle Kurven der Datenbasis und setzt Indizes zurueck
         //! Rueckkehrcode immer true
         BOOL New( void );

         //! liefert als Rueckkehrcode die Anzahl maximal speicherbarer Kurven pro
         //! Datenbasis (default=999)
         int GetMaxScanNumber( void )
         {
                int cn = nMaxCurve - 1;
                return cn;
         };

         //! liefert als Rueckkehrcode den zuletzt aufgetretenen Fehler
         TCFault GetFault( void )
         { return eFault; };

  private:

         //! Anzahl maximal speicherbarer Kurven (def.=1000)
         static const int nMaxCurve;

         //!  zuletzt aufgetretenen Fehler
         TCFault eFault;
         int nMaxCurveIdx;   // indiziert die letzte Kurve
         int nActualIdx;     // indiziert die aktuelle Kurve

         //! Liste von Kurven der Datenbasis
         LPCurve* CurveList;

         //! Kurve enthaelt Werte zu jeder Kurve der Datenbasis
         //! Omegawinkel,Thetawinkel(absoluten winkel) und je nach modus einen
         //! dritten wert (monitorintens., absorberpos.,integrale intens. ,...)
         LPCurve ScanReport;
};

typedef TDataBase FAR * LPDataBase;

#endif
//M_CURVE_H
// __LastLine__
